@@ -24,12 +24,12 @@ for(i in seq_along(all_pkg)) {
 
 # pkgndep results
 saveRDS(lt, file = "all_pkgs.rds", compress = "xz")
-file.copy("all_pkgs.rds", "~/project/development/pkgndep.db/inst/extdata/all_pkgs.rds", overwrite = TRUE
+file.copy("all_pkgs.rds", "~/project/development/pkgndep.github.io/all_pkgs.rds", overwrite = TRUE)
 # lt = readRDS("all_pkgs.rds")
 
 # pkg_db
 saveRDS(pkgndep:::env$pkg_db, file = "pkg_db_snapshot.rds", compress = "xz")
-file.copy("pkg_db_snapshot.rds", "~/project/development/pkgndep.db/inst/extdata/pkg_db_snapshot.rds", overwrite = TRUE)
+file.copy("pkg_db_snapshot.rds", "~/project/development/pkgndep.github.io/pkg_db_snapshot.rds", overwrite = TRUE)
 
 
 ## a data frame that contains various statistics
@@ -38,8 +38,7 @@ df = data.frame(
 	repository = sapply(lt, function(x) x$repository),
 	n_by_strong = sapply(lt, function(x) x$n_by_strong),
 	n_by_all = sapply(lt, function(x) x$n_by_all),
-	n_parents = sapply(lt, function(x) sum(x$which_required)),
-	gini_index = sapply(lt, function(x) gini_index(x$heaviness[x$which_required] + 2))
+	n_parents = sapply(lt, function(x) sum(x$which_required))
 )
 
 df$max_heaviness_from_parents = sapply(lt, function(x) {
@@ -92,12 +91,53 @@ df$max_co_heaviness_from_parents = sapply(lt, function(x) {
 	qqcat("========= @{x$package} ===========\n")
 	m = co_heaviness(x)
 	if(nrow(m) > 1) {
-		max(m[upper.tri(m)])
+		attr(m, "max")
 	} else {
 		0
 	}
 })
 
+df$max_co_heaviness_parents_pair = sapply(lt, function(x) {
+	qqcat("========= @{x$package} ===========\n")
+	m = co_heaviness(x)
+	if(nrow(m) > 1) {
+		paste(attr(m, "max_pair"), collapse = ",")
+	} else {
+		""
+	}
+})
+
+df$max_co_heaviness_parents_pair_type = sapply(lt, function(x) {
+	qqcat("========= @{x$package} ===========\n")
+	m = co_heaviness(x)
+	v = ""
+	if(nrow(m) > 1) {
+		p = attr(m, "max_pair")
+		if(length(p)) {
+			if(is_parent(p[1], p[2])) {
+				v = "parent-child"
+			} else if(is_parent(p[2], p[1])) {
+				v = "parent-child"
+			} else if(is_upstream(p[1], p[2])) {
+				v = "upstream-downstream"
+			} else if(is_parent(p[2], p[1])) {
+				v = "upstream-downstream"
+			} else {
+				up1 = heaviness_from_upstream(p[1])
+				up2 = heaviness_from_upstream(p[2])
+				# the max heaviness of common upstream
+				max = attr(m, "max")
+				cn = intersect(names(up1[up1 > max*0.75]), names(up2[up2 > max*0.75]))
+				if(length(cn)) {
+					v = "have-common-upstream"
+				}
+			}
+		} 
+	}
+	v
+})
+
+
 df$heaviness_on_children = sapply(lt, function(x) {
 	heaviness_on_children(x$package)
 })
@@ -139,7 +179,7 @@ df$adjusted_heaviness_on_downstream = sapply(score, function(x) {
 
 
 ### downstream without direct child packages
-df$heaviness_on_downstream_no_children = sapply(score, function(x) {
+df$heaviness_on_indirect_downstream = sapply(score, function(x) {
 	v = attr(x, "values")
 	children = child_dependency(attr(x, "package"), fields = c("Depends", "Imports", "LinkingTo"))[, 2]
 
@@ -151,7 +191,7 @@ df$heaviness_on_downstream_no_children = sapply(score, function(x) {
 	}
 })
 
-df$n_downstream_no_children = sapply(score, function(x) {
+df$n_indierct_downstream = sapply(score, function(x) {
 	v = attr(x, "values")
 	children = child_dependency(attr(x, "package"), fields = c("Depends", "Imports", "LinkingTo"))[, 2]
 
@@ -159,7 +199,7 @@ df$n_downstream_no_children = sapply(score, function(x) {
 	length(p)
 })
 
-df$hv_downstream_no_children_values = I(lapply(score, function(x) {
+df$hv_indirect_downstream_values = I(lapply(score, function(x) {
 	v = attr(x, "values")
 	children = child_dependency(attr(x, "package"), fields = c("Depends", "Imports", "LinkingTo"))[, 2]
 
@@ -167,7 +207,7 @@ df$hv_downstream_no_children_values = I(lapply(score, function(x) {
 	v[p]
 }))
 
-df$adjusted_heaviness_on_downstream_no_children = sapply(score, function(x) {
+df$adjusted_heaviness_on_indirect_downstream = sapply(score, function(x) {
 	v = attr(x, "values")
 	children = child_dependency(attr(x, "package"), fields = c("Depends", "Imports", "LinkingTo"))[, 2]
 
@@ -182,7 +222,7 @@ df$adjusted_heaviness_on_downstream_no_children = sapply(score, function(x) {
 
 saveRDS(score, file = "pkg_stat_score.rds", compress = "xz")
 saveRDS(df, file = "pkg_stat_snapshot.rds", compress = "xz")
-file.copy("pkg_stat_snapshot.rds", "~/project/development/pkgndep.db/inst/extdata/pkg_stat_snapshot.rds", overwrite = TRUE)
+file.copy("pkg_stat_snapshot.rds", "~/project/development/pkgndep.github.io/pkg_stat_snapshot.rds", overwrite = TRUE)
 
 ######
 
@@ -231,23 +271,21 @@ select_a_for_adjusted_heaviness = function(which = "children", all_a = 0:30, ran
 		d[i-1] = sum(rk > rank_diff)
 	}
 
-	plot(all_a[-1], d, xlab = "value of a", ylab = "sum(abs(rank(v) - rank(prev_v)) > 50)")
-
 	return(data.frame(a = all_a[-1], v = d))
 }
 
 d1 = select_a_for_adjusted_heaviness("children")
-plot(d1[, 1], d1[, 2], xlab = "value of a", ylab = "#{abs(rank(v) - rank(prev_v)) > 50}", main = "Select a for adjusted heaviness on child packages"); abline(v = 10, col = "red")
+plot(d1[, 1], d1[, 2], xlab = "value of a", ylab = "sum(abs(rank(v) - rank(prev_v)) > 50)", main = "Select a for adjusted heaviness on child packages"); abline(v = 10, col = "red")
 
 d2 = select_a_for_adjusted_heaviness("downstream")
-plot(d2[, 1], d2[, 2], xlab = "value of a", ylab = "#{abs(rank(v) - rank(prev_v)) > 50}", main = "Select a for adjusted heaviness on downstream packages"); abline(v = 15, col = "red")
+plot(d2[, 1], d2[, 2], xlab = "value of a", ylab = "sum(abs(rank(v) - rank(prev_v)) > 50)", main = "Select a for adjusted heaviness on downstream packages"); abline(v = 15, col = "red")
 
-d3 = select_a_for_adjusted_heaviness("downstream_no_children")
-plot(d3[, 1], d3[, 2], xlab = "value of a", ylab = "#{abs(rank(v) - rank(prev_v)) > 50}", main = "Select a for adjusted heaviness on downstream packages excluding children"); abline(v = 6, col = "red")
+d3 = select_a_for_adjusted_heaviness("indirect_downstream")
+plot(d3[, 1], d3[, 2], xlab = "value of a", ylab = "sum(abs(rank(v) - rank(prev_v)) > 50)", main = "Select a for adjusted heaviness on downstream packages excluding children"); abline(v = 6, col = "red")
 
 
-saveRDS(list(children = d1, downstream = d2, downstream_no_children = d3), file = "adjusted_heaviness_select_a.rds", compress = "xz")
-file.copy("adjusted_heaviness_select_a.rds", "~/project/development/pkgndep.db/inst/extdata/adjusted_heaviness_select_a.rds", overwrite = TRUE)
+saveRDS(list(children = d1, downstream = d2, indirect_downstream = d3), file = "adjusted_heaviness_select_a.rds", compress = "xz")
+file.copy("adjusted_heaviness_select_a.rds", "~/project/development/pkgndep.github.io/adjusted_heaviness_select_a.rds", overwrite = TRUE)
 
 #### dependency path to all downstream packages
 
@@ -289,7 +327,7 @@ for(package in names(lt)) {
 }
 
 saveRDS(downstream_path_list, file = "pkg_downstream_dependency_path_snapshot.rds", compress = "xz")
-file.copy("pkg_downstream_dependency_path_snapshot.rds", "~/project/development/pkgndep.db/inst/extdata/pkg_downstream_dependency_path_snapshot.rds", overwrite = TRUE)
+file.copy("pkg_downstream_dependency_path_snapshot.rds", "~/project/development/pkgndep.github.io/pkg_downstream_dependency_path_snapshot.rds", overwrite = TRUE)
 
 
 path_list_to_igraph = function(pl) {
