@@ -34,11 +34,13 @@ cat("- Load website components.\n")
 httpd = Rhttpd$new()
 suppressMessages(httpd$start(quiet = TRUE))
 
-httpd$add(app = File$new(system.file("www", "_js", package = "pkgndep")), name = 'js')
-httpd$add(app = File$new(system.file("www", "_css", package = "pkgndep")), name = 'css')
-
-# httpd$add(app = File$new("~/project/development/pkgndep/inst/www/_js"), name = 'js')
-# httpd$add(app = File$new("~/project/development/pkgndep/inst/www/_css"), name = 'css')
+if(identical(unname(Sys.info()[c("sysname", "user")]), c("Darwin", "guz"))) {
+	httpd$add(app = File$new("~/project/development/pkgndep/inst/www/_js"), name = 'js')
+	httpd$add(app = File$new("~/project/development/pkgndep/inst/www/_css"), name = 'css')
+} else {
+	httpd$add(app = File$new(system.file("www", "_js", package = "pkgndep")), name = 'js')
+	httpd$add(app = File$new(system.file("www", "_css", package = "pkgndep")), name = 'css')
+}
 
 request_log = function(page, param) {
 
@@ -95,9 +97,9 @@ httpd$add(name = "main",
 	}
 
 	if(exclude_children && order_by == "adjusted_heaviness_on_downstream") {
-		order_by = "adjusted_heaviness_on_downstream_no_children"
+		order_by = "adjusted_heaviness_on_indirect_downstream"
 	}
-	if(!exclude_children && order_by == "adjusted_heaviness_on_downstream_no_children") {
+	if(!exclude_children && order_by == "adjusted_heaviness_on_indirect_downstream") {
 		order_by = "adjusted_heaviness_on_downstream"
 	}
 
